#include <Arduino.h>
#include <esp_dmx.h>

#include <ESPAsyncE131.h>

#define UNIVERSE 1                      // First DMX Universe to listen for
#define UNIVERSE_COUNT 2                // Total number of Universes to listen for, starting at UNIVERSE

const char ssid[] = "neuland";         // Replace with your SSID
const char passphrase[] = "kreismilliways";   // Replace with your WPA2 passphrase

// ESPAsyncE131 instance with UNIVERSE_COUNT buffer slots
ESPAsyncE131 e131(UNIVERSE_COUNT);

int transmitPin = 21;
int receivePin = 20;
int enablePin = 10;

dmx_port_t dmxPort = 1;

byte data[DMX_PACKET_SIZE];

unsigned long lastUpdate = millis();

void setup() {
  Serial.begin(115200);

  dmx_config_t config = DMX_CONFIG_DEFAULT;
  dmx_personality_t personalities[] = {};
  int personality_count = 0;
  dmx_driver_install(dmxPort, &config, personalities, personality_count);

  dmx_set_pin(dmxPort, transmitPin, receivePin, enablePin);

  delay(3000);

  // Initialize E1.31

  // Make sure you're in station mode    
  WiFi.mode(WIFI_STA);

  Serial.println("");
  Serial.print(F("Connecting to "));
  Serial.print(ssid);
  
  if (passphrase != NULL)
      WiFi.begin(ssid, passphrase);
  else
      WiFi.begin(ssid);
  
  while (WiFi.status() != WL_CONNECTED) {
      delay(500);
      Serial.print(".");
  }
  
  Serial.println("");
  Serial.print(F("Connected with IP: "));
  Serial.println(WiFi.localIP());
  
  // Choose one to begin listening for E1.31 data
  //if (e131.begin(E131_UNICAST))                               // Listen via Unicast
  if (e131.begin(E131_MULTICAST, UNIVERSE, UNIVERSE_COUNT))   // Listen via Multicast
      Serial.println(F("Listening for data..."));
  else 
      Serial.println(F("*** e131.begin failed ***"));
}

void loop() {
  // unsigned long now = millis();
  //Serial.printf("Sending DMX 0x%02X\n", data[1]);


  if (!e131.isEmpty()) {
    e131_packet_t packet;
    e131.pull(&packet);     // Pull packet from ring buffer

    //data[1] = packet.property_values[1];
    
    Serial.printf("Universe %u / %u Channels | Packet#: %u / Errors: %u / CH1: %u\n",
          htons(packet.universe),                 // The Universe for this packet
          htons(packet.property_value_count) - 1, // Start code is ignored, we're interested in dimmer data
          e131.stats.num_packets,                 // Packet counter
          e131.stats.packet_errors,               // Packet error counter
          packet.property_values[1]);             // Dimmer data for Channel 1

    //dmx_wait_sent(dmxPort, DMX_TIMEOUT_TICK);
    dmx_write(dmxPort, packet.property_values, DMX_PACKET_SIZE);
  }
  dmx_send(dmxPort);
}